'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = createActions;

var _immutable = require('immutable');

var _isoFetchStream = require('iso-fetch-stream');

var _isoFetchStream2 = _interopRequireDefault(_isoFetchStream);

var _schema = require('../lib/schema');

var _store = require('./store');

var _store2 = _interopRequireDefault(_store);

var _log = require('../lib/log');

var _query_string = require('../lib/query_string');

var _query_string2 = _interopRequireDefault(_query_string);

var _config = require('../config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var methodDict = {
  create: 'post',
  list: 'get',
  get: 'get',
  update: 'put',
  delete: 'delete',
  search: 'get'
};

var logRoutes = function logRoutes(routes, resource, ns) {
  if (_config2.default.mode === 'development') {
    /* eslint-disable no-console */
    console.groupCollapsed('Rendering Api component for %c' + resource, 'color: green', 'with these routes');
    console.table(Object.keys(routes).reduce(function (memo, key) {
      var val = typeof routes[key] === 'function' ? routes[key]('id', {}) : routes[key];
      var def = typeof val === 'string' ? { method: methodDict[key], route: val } : val;
      var arg = key === 'get' ? 'id' : '';
      return Object.assign(memo, _defineProperty({}, key, Object.assign(def, _defineProperty({}, 'props.' + ns + '...', '$' + key + '(' + arg + ')'))));
    }, {}));
    console.groupEnd();
  }
};

function configureRoutes(key, rootPath) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};


  var routes = Object.assign({
    create: function create() {
      return { method: 'post', route: rootPath };
    },
    list: function list() {
      return { method: 'get', route: rootPath };
    },
    search: function search() {
      return { method: 'get', route: rootPath };
    },
    get: function get(id) {
      return { method: 'get', route: rootPath + '/' + id };
    },
    update: function update(id) {
      return { method: 'put', route: rootPath + '/' + id };
    },
    delete: function _delete(id) {
      return { method: 'delete', route: rootPath + '/' + id };
    }
  }, options.routes || {});

  logRoutes(routes, rootPath, key);

  return {
    getRouteConfig: function getRouteConfig(methodKey) {
      for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }

      var config = routes[methodKey].apply(routes, args);
      return typeof config === 'string' ? { method: methodDict[methodKey], route: config, getData: function getData(data) {
          return data;
        } } : _extends({}, config, {
        method: (config.method || methodDict[methodKey] || 'get').toLowerCase(),
        route: config.route || routes.default || rootPath,
        getData: config.getData || options.getData || function (data) {
          return data;
        }
      });
    }
  };
}

var methodizeResource = function methodizeResource(fetch, props) {
  return function (memo, key) {
    var url = props.url;

    var schema = (0, _schema.getSchema)(props[key]);
    var options = _typeof(props[key]) === 'object' ? props[key] : {};
    var resourceType = schema.title;
    var resourcePath = '/' + resourceType;
    var apiStore = (0, _store2.default)(url, schema);

    // TODO: use some more specific method from store's undo implementation
    var isUndo = function isUndo() {
      return apiStore._getState('_reset');
    };

    var _configureRoutes = configureRoutes(key, resourcePath, options),
        getRouteConfig = _configureRoutes.getRouteConfig;

    var shouldFetch = function shouldFetch(path) {
      return !isUndo() && !apiStore.getPending(path) && !apiStore.getError(path);
    };

    var api = ['get', 'post', 'put', 'delete'].reduce(function (memo, method) {
      return _extends({}, memo, _defineProperty({}, method, function (path) {
        for (var _len2 = arguments.length, args = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
          args[_key2 - 1] = arguments[_key2];
        }

        apiStore.setPending(path, true);
        return fetch[method].apply(fetch, ['' + url + path].concat(args)).then(function (response) {
          apiStore.setPending(path, false);
          apiStore.setError(path, null, { quiet: true });
          return response;
        }, function (err) {
          apiStore.setPending(path, false);
          apiStore.setError(path, err);
          return Promise.reject(err);
        });
      }));
    }, {});

    /**
     * api calls
     */
    var fetchAll = function fetchAll(path) {
      var _getRouteConfig = getRouteConfig('list'),
          defaultRoute = _getRouteConfig.route,
          method = _getRouteConfig.method,
          getData = _getRouteConfig.getData;

      var route = path || defaultRoute;
      if (shouldFetch(route)) {
        return api[method](route).then(function (response) {
          var data = getData(response);
          apiStore.setCollection(data, route);
          return response;
        });
      } else {
        // TODO: store pendingpromise to return here?
      }
    };

    var fetchOne = function fetchOne(id) {
      var _getRouteConfig2 = getRouteConfig('get', id),
          route = _getRouteConfig2.route,
          method = _getRouteConfig2.method,
          getData = _getRouteConfig2.getData;

      if (shouldFetch(route)) {
        return api[method](route).then(function (response) {
          var data = getData(response);
          apiStore.setModel((0, _schema.getIdFromModel)(data), _extends({}, data, { _fetched: true }));
          return response;
        });
      } else {
        // TODO: store pendingpromise to return here?
      }
    };

    var createOne = function createOne(data) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var _getRouteConfig3 = getRouteConfig('create', data),
          route = _getRouteConfig3.route,
          method = _getRouteConfig3.method;

      return api[method](route, data).then(function (data) {
        if (options.refetch !== false) {
          fetchAll();
        } else {
          apiStore.updateCollection(data);
        }
        return data;
      });
    };

    var updateOne = function updateOne(id, data) {
      var _getRouteConfig4 = getRouteConfig('update', id, data),
          route = _getRouteConfig4.route,
          method = _getRouteConfig4.method;

      return api[method](route, data);
    };

    var deleteOne = function deleteOne(id) {
      var _getRouteConfig5 = getRouteConfig('delete', id),
          route = _getRouteConfig5.route,
          method = _getRouteConfig5.method;

      return api[method](route);
    };

    /**/

    var $clear = function $clear(id) {
      return function () {
        return apiStore.setModel(id, null);
      };
    };
    var $reset = function $reset(id) {
      return function () {
        return fetchOne(id);
      };
    };

    var optimisticDelete = function optimisticDelete(id) {
      var undoDelete = apiStore.deleteModel(id);
      if (undoDelete.length) {
        return deleteOne(id).catch(function (err) {
          (0, _log.logError)('Failed to delete ' + id, err);
          undoDelete.forEach(function (undo) {
            return undo();
          });
          return Promise.reject(err);
        });
      } else {
        return Promise.reject(new Error(404));
      }
    };

    var $delete = function $delete(id) {
      return function () {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
        return options.optimistic === false || !apiStore.getModel(id) ? deleteOne(id).then(function () {
          return apiStore.deleteModel(id);
        }) : optimisticDelete(id);
      };
    };

    var optimisticUpdate = function optimisticUpdate(id, vals) {
      var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

      var placeholder = typeof options.optimistic === 'function' ? options.optimistic(_extends({ id: id }, vals)) : vals;
      var undoUpdate = apiStore.updateModel(id, placeholder);
      if (undoUpdate.length) {
        return updateOne(id, vals).catch(function (err) {
          (0, _log.logError)('Failed to update ' + id + ' with vals', vals, err);
          undoUpdate.forEach(function (undo) {
            return undo();
          });
          return Promise.reject(err);
        });
      } else {
        return Promise.reject(new Error(404));
      }
    };

    var $update = function $update(id) {
      return function (vals) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        return options.optimistic === false || !apiStore.getModel(id) ? updateOne(id, vals).then(function (response) {
          var _getRouteConfig6 = getRouteConfig('update', id, vals),
              getData = _getRouteConfig6.getData;

          apiStore.updateModel(id, getData(response));
          return response;
        }) : optimisticUpdate(id, vals, options);
      };
    };

    var getPlaceholder = function getPlaceholder() {
      var path = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
      var dataType = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _immutable.List;

      var placeholder = dataType();
      placeholder.$isPending = true;
      placeholder.$error = path ? apiStore.getError(path) : null;
      placeholder.$clear = function () {
        return undefined;
      };
      placeholder.$reset = function () {
        return Promise.reject('no data to reset');
      };
      return placeholder;
    };

    var addRestMethods = function addRestMethods(model) {
      var id = (0, _schema.getIdFromModel)(model);
      model.$delete = $delete(id);
      model.$update = $update(id);
      model.$clear = $clear(id);
      model.$reset = $reset(id);
      return model;
    };

    var $list = function $list(params) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var _getRouteConfig7 = getRouteConfig('list', params),
          route = _getRouteConfig7.route;

      var path = route + (params ? '?' + (0, _query_string2.default)(params) : '');
      var collection = apiStore.getCollection(path);
      if (options.reset || !collection) {
        fetchAll(path);
        return getPlaceholder(path);
      } else {
        var mapped = collection.map(addRestMethods);
        mapped.$clear = function () {
          return apiStore.clearCollection(path);
        };
        mapped.$reset = function () {
          mapped.$clear();
          $list(params);
        };
        return mapped;
      }
    };

    var main = function main(params) {
      return $list(params);
    };

    main.$list = $list;

    main.$get = function (id) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var model = apiStore.getModel(id);
      if (options.reset || !model || !model.get('_fetched')) {
        var _getRouteConfig8 = getRouteConfig('get', id),
            route = _getRouteConfig8.route;

        fetchOne(id);
        var placeholder = getPlaceholder(route, _immutable.Map);
        placeholder.$delete = $delete(id);
        placeholder.$update = $update(id);
        return placeholder;
      } else {
        return addRestMethods(model);
      }
    };

    main.$clearAll = apiStore.clearAll;
    main.$reset = fetchAll;

    var optimisticCreate = function optimisticCreate(data) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      if (typeof options.optimistic === 'function') {
        var nextState = apiStore.getState().withMutations(function (map) {
          return options.optimistic(map, data);
        });
        if (nextState) apiStore.setState(nextState);
        return createOne(data, options);
      } else {
        (0, _log.logWarn)('Optimistic creates must receive a function that updates the state with the optimistic data. The create will proceed pessimistically.');
        return createOne(data, options);
      }
    };

    main.$create = function (data) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      return !options.optimistic ? createOne(data, options) : optimisticCreate(data, options);
    };

    main.$search = function (params) {
      var _getRouteConfig9 = getRouteConfig('search', params),
          method = _getRouteConfig9.method,
          route = _getRouteConfig9.route;

      var fullRoute = method === 'get' ? route + ('?' + (0, _query_string2.default)(params)) : route;
      if (shouldFetch(fullRoute)) {
        var promise = method === 'get' ? api.get(fullRoute) : api[method](fullRoute, params);

        return promise.then(function (data) {
          apiStore.setCollection(data, fullRoute);
          return data;
        });
      }
      // TODO: store promise as pending value so it can be used on repeat
      // calls
      return Promise.resolve('pending');
    };

    main.$search.results = function (params) {
      var _getRouteConfig10 = getRouteConfig('search', params),
          route = _getRouteConfig10.route,
          method = _getRouteConfig10.method;

      var fullRoute = method === 'get' ? route + ('?' + (0, _query_string2.default)(params)) : route;
      var collection = apiStore.getCollection(fullRoute);
      if (collection) {
        var mapped = collection.map(addRestMethods);
        mapped.$clear = function () {
          return apiStore.clearCollection(fullRoute);
        };
        mapped.$reset = function () {
          main.$clear();
          main.$search(params);
        };
        return mapped;
      } else {
        var placeholder = getPlaceholder(fullRoute);
        placeholder.$isPending = !!apiStore.getPending(fullRoute);
        return placeholder;
      }
    };

    // hang standard api methods off of .api so devs can construct
    // non-standard paths
    main.api = ['delete', '$get', 'get', 'post', 'put', 'stream'].reduce(function (memo, method) {
      return _extends({}, memo, _defineProperty({}, method, function (path) {
        for (var _len3 = arguments.length, args = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
          args[_key3 - 1] = arguments[_key3];
        }

        var route = resourcePath + path;
        if (method === '$get') {
          var cache = apiStore.getRequestsData(route);
          if (cache) {
            return _immutable.List.isList(cache) ? apiStore.getCollection(route).map(addRestMethods) : cache;
          } else {
            if (shouldFetch(route)) {
              api.get.apply(api, [route].concat(args)).then(function (data) {
                if (Array.isArray(data)) {
                  apiStore.setCollection(data, route);
                } else {
                  apiStore.setRequestsData(route, data);
                }
                return data;
              });
            }
            return getPlaceholder(route);
          }
        } else {
          return api[method].apply(api, [route].concat(args));
        }
      }));
    }, {});

    // add custom methods specified by user in routes config

    var fetchCustom = function fetchCustom(config) {
      return api[config.method](config.route, config.body).then(function (response) {
        var data = config.getData(response);
        apiStore.setRequestsData(config.route, data);
        return response;
      });
    };

    Object.keys(options.routes || {}).forEach(function (key) {
      if (!methodDict[key]) {
        main['$' + key] = function () {
          for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
            args[_key4] = arguments[_key4];
          }

          var config = getRouteConfig.apply(undefined, [key].concat(args));
          if (config.usesCache) {
            var cache = apiStore.getRequestsData(config.route);
            if (cache) {
              cache.$clear = function () {
                return apiStore.setRequestsData(config.route, null);
              };
              cache.$reset = function () {
                cache.$clear();
                fetchCustom(config);
              };
              return cache;
            } else {
              if (shouldFetch(config.route)) fetchCustom(config);
              return getPlaceholder(config.route);
            }
          } else {
            return api[config.method](config.route, config.body).then(function (response) {
              return config.getData(response);
            });
          }
        };
      }
    });

    main._schema = schema;
    main._resourceType = resourceType;
    main.getState = apiStore.getState;

    memo[key] = main;
    if (typeof window !== 'undefined') {
      window.jetset = window.jetset || {};
      window.jetset[key] = main;
    }

    return memo;
  };
};

function createActions(props) {

  var fetchOptions = props.credentials ? { credentials: props.credentials } : {};

  if (props.auth || props.authorization) {
    fetchOptions.headers = { Authorization: props.auth || props.authorization };
  }

  var fetch = (0, _isoFetchStream2.default)(fetchOptions);

  return Object.keys(props).filter((0, _schema.isSchema)(props)).reduce(methodizeResource(fetch, props), {});
}