'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.default = createActions;

var _immutable = require('immutable');

var _isoFetchStream = require('iso-fetch-stream');

var _isoFetchStream2 = _interopRequireDefault(_isoFetchStream);

var _routes = require('./routes');

var _routes2 = _interopRequireDefault(_routes);

var _store = require('./store');

var _store2 = _interopRequireDefault(_store);

var _methods = require('./methods');

var _methods2 = _interopRequireDefault(_methods);

var _log = require('../lib/log');

var _query_string = require('../lib/query_string');

var _query_string2 = _interopRequireDefault(_query_string);

var _schema = require('../lib/schema');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var methodizeResource = function methodizeResource(fetch, props) {
  return function (memo, key) {
    var url = props.url;

    var schema = (0, _schema.getSchema)(props[key]);
    var options = _typeof(props[key]) === 'object' ? props[key] : {};
    var resourceType = schema.title;
    var resourcePath = '/' + resourceType;

    // api-specific helpers for working with the state tree
    var apiStore = (0, _store2.default)(url, schema);

    // set up all routes

    var _configureRoutes = (0, _routes2.default)(key, resourcePath, options),
        getRouteConfig = _configureRoutes.getRouteConfig,
        isCustomRoute = _configureRoutes.isCustomRoute;

    // xhr + fetch state helpers


    var api = (0, _methods2.default)(fetch, apiStore, getRouteConfig);

    // placeholder to return while cacheable fetches are pending
    var getPlaceholder = function getPlaceholder() {
      var path = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
      var dataType = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : _immutable.List;

      var placeholder = dataType();
      placeholder.$isPending = true;
      placeholder.$error = path ? apiStore.getError(path) : null;
      placeholder.$clear = function () {
        return undefined;
      };
      placeholder.$reset = function () {
        return Promise.reject('no data to reset');
      };
      return placeholder;
    };

    // LIST

    var $list = function $list(params) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var _getRouteConfig = getRouteConfig('list', params),
          route = _getRouteConfig.route;

      var path = route + (params ? '?' + (0, _query_string2.default)(params) : '');
      var collection = apiStore.getCollection(path);
      if (options.reset || !collection) {
        api.fetchAll(path);
        return getPlaceholder(path);
      } else {
        var mapped = collection.map(addRestMethods);
        mapped.$clear = function () {
          return apiStore.clearCollection(path);
        };
        mapped.$reset = function () {
          mapped.$clear();
          $list(params);
        };
        return mapped;
      }
    };

    // SEARCH

    var $search = function $search(params) {
      var _getRouteConfig2 = getRouteConfig('search', params),
          method = _getRouteConfig2.method,
          route = _getRouteConfig2.route,
          getData = _getRouteConfig2.getData;

      var fullRoute = method === 'get' ? route + ('?' + (0, _query_string2.default)(params)) : route;
      if (api.shouldFetch(fullRoute)) {
        var promise = method === 'get' ? api.get(fullRoute) : api[method](fullRoute, params);

        return promise.then(function (data) {
          apiStore.setCollection(getData(data), fullRoute);
          return data;
        });
      }
      // TODO: store promise as pending value so it can be used on repeat
      // calls
      return Promise.resolve('pending');
    };

    $search.results = function (params) {
      var _getRouteConfig3 = getRouteConfig('search', params),
          route = _getRouteConfig3.route,
          method = _getRouteConfig3.method;

      var fullRoute = method === 'get' ? route + ('?' + (0, _query_string2.default)(params)) : route;
      var collection = apiStore.getCollection(fullRoute);
      if (collection) {
        var mapped = collection.map(addRestMethods);
        mapped.$clear = function () {
          return apiStore.clearCollection(fullRoute);
        };
        mapped.$reset = function () {
          $clear();
          $search(params);
        };
        return mapped;
      } else {
        var placeholder = getPlaceholder(fullRoute);
        placeholder.$isPending = !!apiStore.getPending(fullRoute);
        return placeholder;
      }
    };

    // CREATE

    var optimisticCreate = function optimisticCreate(data) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      if (typeof options.optimistic === 'function') {
        var nextState = apiStore.getState().withMutations(function (map) {
          return options.optimistic(map, data);
        });
        if (nextState) apiStore.setState(nextState);
        return api.createOne(data, options);
      } else {
        (0, _log.logWarn)('Optimistic creates must receive a function that updates the state with the optimistic data. The create will proceed pessimistically.');
        return api.createOne(data, options);
      }
    };

    var $create = function $create(data) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      return !options.optimistic ? api.createOne(data, options) : optimisticCreate(data, options);
    };

    // GET

    var $get = function $get(id) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var model = apiStore.getModel(id);
      if (options.reset || !model || !model.get('_fetched')) {
        var _getRouteConfig4 = getRouteConfig('get', id),
            route = _getRouteConfig4.route;

        api.fetchOne(id);
        var placeholder = getPlaceholder(route, _immutable.Map);
        placeholder.$delete = $delete(id);
        placeholder.$update = $update(id);
        return placeholder;
      } else {
        return addRestMethods(model);
      }
    };

    // DELETE 

    var optimisticDelete = function optimisticDelete(id) {
      var undoDelete = apiStore.deleteModel(id);
      if (undoDelete.length) {
        return api.deleteOne(id).catch(function (err) {
          (0, _log.logError)('Failed to delete ' + id, err);
          undoDelete.forEach(function (undo) {
            return undo();
          });
          return Promise.reject(err);
        });
      } else {
        return Promise.reject(new Error(404));
      }
    };

    var $delete = function $delete(id) {
      return function () {
        var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
        return options.optimistic === false || !apiStore.getModel(id) ? api.deleteOne(id).then(function () {
          return apiStore.deleteModel(id);
        }) : optimisticDelete(id);
      };
    };

    // UPDATE

    var optimisticUpdate = function optimisticUpdate(id, vals) {
      var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

      var placeholder = typeof options.optimistic === 'function' ? options.optimistic(_extends({ id: id }, vals)) : vals;
      var undoUpdate = apiStore.updateModel(id, placeholder);
      if (undoUpdate.length) {
        return api.updateOne(id, vals).then(function (response) {
          var _getRouteConfig5 = getRouteConfig('update', id, vals),
              getData = _getRouteConfig5.getData;

          apiStore.updateModel(id, getData(response));
          return response;
        }).catch(function (err) {
          (0, _log.logError)('Failed to update ' + id + ' with vals', vals, err);
          undoUpdate.forEach(function (undo) {
            return undo();
          });
          return Promise.reject(err);
        });
      } else {
        return Promise.reject(new Error(404));
      }
    };

    var $update = function $update(id) {
      return function (vals) {
        var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
        return options.optimistic === false || !apiStore.getModel(id) ? api.updateOne(id, vals).then(function (response) {
          var _getRouteConfig6 = getRouteConfig('update', id, vals),
              getData = _getRouteConfig6.getData;

          apiStore.updateModel(id, getData(response));
          return response;
        }) : optimisticUpdate(id, vals, options);
      };
    };

    // CACHE management for models

    var $clear = function $clear(id) {
      return function () {
        return apiStore.setModel(id, null);
      };
    };
    var $reset = function $reset(id) {
      return function () {
        return api.fetchOne(id);
      };
    };

    var addRestMethods = function addRestMethods(model) {
      var id = (0, _schema.getIdFromModel)(model);
      model.$delete = $delete(id);
      model.$update = $update(id);
      model.$clear = $clear(id);
      model.$reset = $reset(id);
      return model;
    };

    // this is the function/object that gets returned and namespaced according to
    // the prop passed in by the user
    var main = function main(params) {
      return $list(params);
    };
    main.$list = $list;
    main.$search = $search;
    main.$create = $create;
    main.$get = $get;
    main.$clearAll = apiStore.clearAll;
    main.$reset = api.fetchAll;

    main.subscribePath = apiStore.subscribePath;
    main.getState = apiStore.getState;

    // hang standard api methods off of .api so devs can construct
    // non-standard paths
    main.api = ['delete', '$get', 'get', 'post', 'put', 'stream'].reduce(function (memo, method) {
      return _extends({}, memo, _defineProperty({}, method, function (path) {
        for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          args[_key - 1] = arguments[_key];
        }

        var route = resourcePath + path;
        if (method === '$get') {
          var cache = apiStore.getRequestsData(route);
          if (cache) {
            return _immutable.List.isList(cache) ? apiStore.getCollection(route).map(addRestMethods) : cache;
          } else {
            if (api.shouldFetch(route)) {
              api.get.apply(api, [route].concat(args)).then(function (data) {
                if (Array.isArray(data)) {
                  apiStore.setCollection(data, route);
                } else {
                  apiStore.setRequestsData(route, data);
                }
                return data;
              });
            }
            return getPlaceholder(route);
          }
        } else {
          return api[method].apply(api, [route].concat(args));
        }
      }));
    }, {});

    // add custom methods specified by user in routes config

    var fetchCacheable = function fetchCacheable(config) {
      var cache = apiStore.getRequestsData(config.route);
      if (cache) {
        cache.$clear = function () {
          return apiStore.setRequestsData(config.route, null);
        };
        cache.$reset = function () {
          cache.$clear();
          api.custom(config);
        };
        return cache;
      } else {
        if (api.shouldFetch(config.route)) api.custom(config);
        return getPlaceholder(config.route);
      }
    };

    var fetchNonCacheable = function fetchNonCacheable(config) {
      return api[config.method](config.route, config.body).then(function (response) {
        return config.getData(response);
      });
    };

    Object.keys(options.routes || {}).forEach(function (key) {
      if (isCustomRoute(key)) {
        main['$' + key] = function () {
          for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
            args[_key2] = arguments[_key2];
          }

          var config = getRouteConfig.apply(undefined, [key].concat(args));
          return config.usesCache ? fetchCacheable(config) : fetchNonCacheable(config);
        };
      }
    });

    memo[key] = main;

    return memo;
  };
};

function createActions(props) {

  var fetchOptions = props.credentials ? { credentials: props.credentials } : {};

  if (props.auth || props.authorization) {
    fetchOptions.headers = { Authorization: props.auth || props.authorization };
  }

  fetchOptions.makeUrl = function (path) {
    return props.url + path;
  };

  var fetch = (0, _isoFetchStream2.default)(fetchOptions);

  return Object.keys(props).filter((0, _schema.isSchema)(props)).reduce(methodizeResource(fetch, props), {});
}