'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = initApiMethods;

var _schema = require('../lib/schema');

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function initApiMethods(fetch, store, getRouteConfig) {

  // TODO: use some more specific method from store's undo implementation
  var isUndo = function isUndo() {
    return store._getState('_reset');
  };
  var shouldFetch = function shouldFetch(path) {
    return !isUndo() && !store.getPending(path) && !store.getError(path);
  };

  var api = ['get', 'post', 'put', 'delete'].reduce(function (memo, method) {
    return _extends({}, memo, _defineProperty({}, method, function (path) {
      for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }

      store.setPending(path, true);
      return fetch[method].apply(fetch, [path].concat(args)).then(function (response) {
        store.setPending(path, false);
        store.setError(path, null, { quiet: true });
        return response;
      }, function (err) {
        store.setPending(path, false);
        store.setError(path, err);
        return Promise.reject(err);
      });
    }));
  }, {});

  var methods = _extends({

    fetchAll: function fetchAll(path) {
      var _getRouteConfig = getRouteConfig('list'),
          defaultRoute = _getRouteConfig.route,
          method = _getRouteConfig.method,
          getData = _getRouteConfig.getData;

      var route = path || defaultRoute;
      if (shouldFetch(route)) {
        return api[method](route).then(function (response) {
          var data = getData(response);
          store.setCollection(data, route);
          return response;
        });
      } else {
        // TODO: store pendingpromise to return here?
      }
    },

    fetchOne: function fetchOne(id) {
      var _getRouteConfig2 = getRouteConfig('get', id),
          route = _getRouteConfig2.route,
          method = _getRouteConfig2.method,
          getData = _getRouteConfig2.getData;

      if (shouldFetch(route)) {
        return api[method](route).then(function (response) {
          var data = getData(response);
          store.setModel((0, _schema.getIdFromModel)(data), _extends({}, data, { _fetched: true }));
          return response;
        });
      } else {
        // TODO: store pendingpromise to return here?
      }
    },

    createOne: function createOne(data) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      var _getRouteConfig3 = getRouteConfig('create', data),
          route = _getRouteConfig3.route,
          method = _getRouteConfig3.method,
          getData = _getRouteConfig3.getData;

      return api[method](route, data).then(function (data) {
        if (options.refetch !== false) {
          methods.fetchAll();
        } else {
          store.updateCollection(getData(data));
        }
        return data;
      });
    },

    updateOne: function updateOne(id, data) {
      var _getRouteConfig4 = getRouteConfig('update', id, data),
          route = _getRouteConfig4.route,
          method = _getRouteConfig4.method;

      return api[method](route, data);
    },

    deleteOne: function deleteOne(id) {
      var _getRouteConfig5 = getRouteConfig('delete', id),
          route = _getRouteConfig5.route,
          method = _getRouteConfig5.method;

      return api[method](route);
    },

    custom: function custom(config) {
      return api[config.method](config.route, config.body).then(function (response) {
        var data = config.getData(response);
        store.setRequestsData(config.route, data);
        return response;
      });
    },

    shouldFetch: shouldFetch
  }, api);

  return methods;
}