(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', '../config'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('../config'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.config);
    global.routes = mod.exports;
  }
})(this, function (exports, _config) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.default = configureRoutes;

  var _config2 = _interopRequireDefault(_config);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  var _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  var methodDict = {
    create: 'post',
    list: 'get',
    get: 'get',
    update: 'put',
    delete: 'delete',
    search: 'get'
  };

  var logRoutes = function logRoutes(routes, resource, ns) {
    if (_config2.default.mode === 'development') {
      /* eslint-disable no-console */
      console.groupCollapsed('Rendering Api component for %c' + resource, 'color: green', 'with these routes');
      console.table(Object.keys(routes).reduce(function (memo, key) {
        var val = typeof routes[key] === 'function' ? routes[key]('id', {}) : routes[key];
        var def = typeof val === 'string' ? { method: methodDict[key], route: val } : val;
        var arg = key === 'get' ? 'id' : '';
        return Object.assign(memo, _defineProperty({}, key, Object.assign(def, _defineProperty({}, 'props.' + ns + '...', '$' + key + '(' + arg + ')'))));
      }, {}));
      console.groupEnd();
    }
  };

  function configureRoutes(key, rootPath) {
    var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};


    var routes = Object.assign({
      create: function create() {
        return { method: 'post', route: rootPath };
      },
      list: function list() {
        return { method: 'get', route: rootPath };
      },
      search: function search() {
        return { method: 'get', route: rootPath };
      },
      get: function get(id) {
        return { method: 'get', route: rootPath + '/' + id };
      },
      update: function update(id) {
        return { method: 'put', route: rootPath + '/' + id };
      },
      delete: function _delete(id) {
        return { method: 'delete', route: rootPath + '/' + id };
      }
    }, options.routes || {});

    logRoutes(routes, rootPath, key);

    return {
      isCustomRoute: function isCustomRoute(key) {
        return !methodDict[key];
      },
      getRouteConfig: function getRouteConfig(methodKey) {
        for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          args[_key - 1] = arguments[_key];
        }

        var config = routes[methodKey].apply(routes, args);
        return typeof config === 'string' ? { method: methodDict[methodKey], route: config, getData: options.getData || function (data) {
            return data;
          } } : _extends({}, config, {
          method: (config.method || methodDict[methodKey] || 'get').toLowerCase(),
          route: config.route || routes.default || rootPath,
          getData: config.getData || options.getData || function (data) {
            return data;
          }
        });
      }
    };
  }
});